/*
 * Instructions1.cpp: Python bindings for the LLVM compiler infrastructure
 * Copyright (c) 2004, 2005 Jarno Seppnen <--jseppanen@gmail.com-->.
 * Distributed under the Boost Software License, Version 1.0. (See
 * accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#include <boost/python.hpp>
#include <boost/noncopyable.hpp>
#include <llvm/BasicBlock.h>
#include <llvm/Constant.h>
#include <llvm/Constants.h>
#include <llvm/DerivedTypes.h>
#include <llvm/Function.h>
#include <llvm/Instructions.h>
#include <llvm/Type.h>
#include <llvm/Value.h>

using namespace boost::python;

// Instruction constructor helpers
#include "inst_init.h"

namespace llvm { namespace Python {

void wrap_AllocationInst()
{
    // Name C++ overloads
    const Value* (AllocationInst::*getArraySize)() const = &AllocationInst::getArraySize;
    bool (*classof_AllocationInst)(const AllocationInst*) = &AllocationInst::classof;
    bool (*classof_Instruction)(const Instruction*) = &AllocationInst::classof;
    bool (*classof_Value)(const Value*) = &AllocationInst::classof;

    class_<AllocationInst, boost::noncopyable, bases<Instruction> >
      ("AllocationInst", no_init)
        .def("isArrayAllocation", &AllocationInst::isArrayAllocation)
        .def("getArraySize", getArraySize,
             return_internal_reference<>())
        .def("getType", &AllocationInst::getType,
             return_internal_reference<>())
        .def("getAllocatedType", &AllocationInst::getAllocatedType,
             return_internal_reference<>())
        .def("clone", &AllocationInst::clone,
             return_value_policy<manage_new_object>())
        .def("classof", classof_AllocationInst)
        .def("classof", classof_Instruction)
        .def("classof", classof_Value)
        .staticmethod("classof")
        ;
}

template MallocInst* InstInitFn4(const Type*, Value*, const std::string&, Instruction*);
template MallocInst* InstInitFn4(const Type*, Value*, const std::string&, BasicBlock*);

void wrap_MallocInst()
{
    // Name C++ overloads
    bool (*classof_MallocInst)(const MallocInst*) = &MallocInst::classof;
    bool (*classof_Instruction)(const Instruction*) = &MallocInst::classof;
    bool (*classof_Value)(const Value*) = &MallocInst::classof;

    class_<MallocInst, boost::noncopyable, bases<AllocationInst> >
        ("MallocInst", init<const Type*, optional<Value*, const std::string> >())
        .def("init", INST_INIT_DEF4(MallocInst, const Type*, Value*, const std::string&, Instruction*))
        .def("init", INST_INIT_DEF4(MallocInst, const Type*, Value*, const std::string&, BasicBlock*))
        .staticmethod("init")

        .def("clone", &MallocInst::clone,
             return_value_policy<manage_new_object>())
        .def("classof", classof_MallocInst)
        .def("classof", classof_Instruction)
        .def("classof", classof_Value)
        .staticmethod("classof")
        ;
}

template AllocaInst* InstInitFn4(const Type*, Value*, const std::string&, Instruction*);
template AllocaInst* InstInitFn4(const Type*, Value*, const std::string&, BasicBlock*);

void wrap_AllocaInst()
{
    // Name C++ overloads
    bool (*classof_AllocaInst)(const AllocaInst*) = &AllocaInst::classof;
    bool (*classof_Instruction)(const Instruction*) = &AllocaInst::classof;
    bool (*classof_Value)(const Value*) = &AllocaInst::classof;

    class_<AllocaInst, boost::noncopyable, bases<AllocationInst> >
        ("AllocaInst", init<const Type*, optional<Value*, const std::string> >())
        .def("init", INST_INIT_DEF4(AllocaInst, const Type*, Value*, const std::string&, Instruction*))
        .def("init", INST_INIT_DEF4(AllocaInst, const Type*, Value*, const std::string&, BasicBlock*))
        .staticmethod("init")

        .def("clone", &AllocaInst::clone,
             return_value_policy<manage_new_object>())
        .def("classof", classof_AllocaInst)
        .def("classof", classof_Instruction)
        .def("classof", classof_Value)
        .staticmethod("classof")
        ;
}

template FreeInst* InstInitFn2(Value*, Instruction*);
template FreeInst* InstInitFn2(Value*, BasicBlock*);

void wrap_FreeInst()
{
    // Name C++ overloads
    bool (*classof_FreeInst)(const FreeInst*) = &FreeInst::classof;
    bool (*classof_Instruction)(const Instruction*) = &FreeInst::classof;
    bool (*classof_Value)(const Value*) = &FreeInst::classof;

    class_<FreeInst, boost::noncopyable, bases<Instruction> >
        ("FreeInst", init<Value*>())
        .def("init", INST_INIT_DEF2(FreeInst, Value*, Instruction*))
        .def("init", INST_INIT_DEF2(FreeInst, Value*, BasicBlock*))
        .staticmethod("init")

        .def("clone", &FreeInst::clone,
             return_value_policy<manage_new_object>())
        .def("classof", classof_FreeInst)
        .def("classof", classof_Instruction)
        .def("classof", classof_Value)
        .staticmethod("classof")
        ;
}

template LoadInst* InstInitFn3(Value*, const std::string&, Instruction*);
template LoadInst* InstInitFn3(Value*, const std::string&, BasicBlock*);
template LoadInst* InstInitFn4(Value*, const std::string&, bool, Instruction*);
template LoadInst* InstInitFn4(Value*, const std::string&, bool, BasicBlock*);

void wrap_LoadInst()
{
    // Name C++ overloads
    const Value* (LoadInst::*getPointerOperand)() const = &LoadInst::getPointerOperand;
    bool (*classof_LoadInst)(const LoadInst*) = &LoadInst::classof;
    bool (*classof_Instruction)(const Instruction*) = &LoadInst::classof;
    bool (*classof_Value)(const Value*) = &LoadInst::classof;

    class_<LoadInst, boost::noncopyable, bases<Instruction> >
        ("LoadInst", init<Value*, optional<const std::string, bool> >())
        .def("init", INST_INIT_DEF3(LoadInst, Value*, const std::string&, Instruction*))
        .def("init", INST_INIT_DEF3(LoadInst, Value*, const std::string&, BasicBlock*))
        .def("init", INST_INIT_DEF4(LoadInst, Value*, const std::string&, bool, Instruction*))
        .def("init", INST_INIT_DEF4(LoadInst, Value*, const std::string&, bool, BasicBlock*))
        .staticmethod("init")

        .def("isVolatile", &LoadInst::isVolatile)
        .def("setVolatile", &LoadInst::setVolatile)
        .def("clone", &LoadInst::clone,
             return_value_policy<manage_new_object>())
        .def("mayWriteToMemory", &LoadInst::mayWriteToMemory)
        .def("getPointerOperand", getPointerOperand,
             return_internal_reference<>())
        .def("getPointerOperandIndex", &LoadInst::getPointerOperandIndex)
        .staticmethod("getPointerOperandIndex")
        .def("classof", classof_LoadInst)
        .def("classof", classof_Instruction)
        .def("classof", classof_Value)
        .staticmethod("classof")
        ;
}

template StoreInst* InstInitFn3(Value*, Value*, Instruction*);
template StoreInst* InstInitFn3(Value*, Value*, BasicBlock*);
template StoreInst* InstInitFn4(Value*, Value*, bool, Instruction*);
template StoreInst* InstInitFn4(Value*, Value*, bool, BasicBlock*);

void wrap_StoreInst()
{
    // Name C++ overloads
    const Value* (StoreInst::*getPointerOperand)() const = &StoreInst::getPointerOperand;
    bool (*classof_StoreInst)(const StoreInst*) = &StoreInst::classof;
    bool (*classof_Instruction)(const Instruction*) = &StoreInst::classof;
    bool (*classof_Value)(const Value*) = &StoreInst::classof;

    class_<StoreInst, boost::noncopyable, bases<Instruction> >
        ("StoreInst", init<Value*, Value*, optional<bool> >())
        .def("init", INST_INIT_DEF3(StoreInst, Value*, Value*, Instruction*))
        .def("init", INST_INIT_DEF3(StoreInst, Value*, Value*, BasicBlock*))
        .def("init", INST_INIT_DEF4(StoreInst, Value*, Value*, bool, Instruction*))
        .def("init", INST_INIT_DEF4(StoreInst, Value*, Value*, bool, BasicBlock*))
        .staticmethod("init")

        .def("isVolatile", &StoreInst::isVolatile)
        .def("setVolatile", &StoreInst::setVolatile)
        .def("clone", &StoreInst::clone,
             return_value_policy<manage_new_object>())
        .def("mayWriteToMemory", &StoreInst::mayWriteToMemory)
        .def("getPointerOperand", getPointerOperand,
             return_internal_reference<>())
        .def("getPointerOperandIndex", &StoreInst::getPointerOperandIndex)
        .staticmethod("getPointerOperandIndex")
        .def("classof", classof_StoreInst)
        .def("classof", classof_Instruction)
        .def("classof", classof_Value)
        .staticmethod("classof")
        ;
}

template GetElementPtrInst* InstInitFn4(Value*, const std::vector<Value*>&, const std::string&, Instruction*);
template GetElementPtrInst* InstInitFn4(Value*, const std::vector<Value*>&, const std::string&, BasicBlock*);
template GetElementPtrInst* InstInitFn5(Value*, Value*, Value*, const std::string&, Instruction*);
template GetElementPtrInst* InstInitFn5(Value*, Value*, Value*, const std::string&, BasicBlock*);

void wrap_GetElementPtrInst()
{
    // Name C++ overloads
    const Type* (*getIndexedType)(const Type*, const std::vector<Value*>&, bool) = &GetElementPtrInst::getIndexedType;
    const Type* (*getIndexedType2)(const Type*, Value*, Value*, bool) = &GetElementPtrInst::getIndexedType;
    GetElementPtrInst::const_op_iterator (GetElementPtrInst::*idx_begin)() const = &GetElementPtrInst::idx_begin;
    GetElementPtrInst::const_op_iterator (GetElementPtrInst::*idx_end)() const = &GetElementPtrInst::idx_end;
    const Value* (GetElementPtrInst::*getPointerOperand)() const = &GetElementPtrInst::getPointerOperand;
    bool (*classof_GetElementPtrInst)(const GetElementPtrInst*) = &GetElementPtrInst::classof;
    bool (*classof_Instruction)(const Instruction*) = &GetElementPtrInst::classof;
    bool (*classof_Value)(const Value*) = &GetElementPtrInst::classof;

    class_<GetElementPtrInst, boost::noncopyable, bases<Instruction> >
        ("GetElementPtrInst", init<Value*, const std::vector<Value*>&,
                                   optional<const std::string> >())
        .def(init<Value*, Value*, Value*,
                  optional<const std::string> >())
        .def("init", INST_INIT_DEF4(GetElementPtrInst, Value*, const std::vector<Value*>&, const std::string&, Instruction*))
        .def("init", INST_INIT_DEF4(GetElementPtrInst, Value*, const std::vector<Value*>&, const std::string&, BasicBlock*))
        .def("init", INST_INIT_DEF5(GetElementPtrInst, Value*, Value*, Value*, const std::string&, Instruction*))
        .def("init", INST_INIT_DEF5(GetElementPtrInst, Value*, Value*, Value*, const std::string&, BasicBlock*))
        .staticmethod("init")

        .def("clone", &GetElementPtrInst::clone,
             return_value_policy<manage_new_object>())
        .def("getType", &GetElementPtrInst::getType,
             return_internal_reference<>())
        .def("getIndexedType", getIndexedType,
             return_internal_reference<>())
        .def("getIndexedType", getIndexedType2,
             return_internal_reference<>())
        .staticmethod("getIndexedType")
        .add_property("indices", range(idx_begin, idx_end))
        //.def("idx_begin", idx_begin)
        //.def("idx_end", idx_end)
        .def("getPointerOperand", getPointerOperand,
             return_internal_reference<>())
        .def("getPointerOperandIndex", &GetElementPtrInst::getPointerOperandIndex)
        .staticmethod("getPointerOperandIndex")
        .def("getNumIndices", &GetElementPtrInst::getNumIndices)
        .def("hasIndices", &GetElementPtrInst::hasIndices)
        .def("classof", classof_GetElementPtrInst)
        .def("classof", classof_Instruction)
        .def("classof", classof_Value)
        .staticmethod("classof")
        ;
}

template SetCondInst* InstInitFn5(Instruction::BinaryOps, Value*, Value*, const std::string&, Instruction*);
template SetCondInst* InstInitFn5(Instruction::BinaryOps, Value*, Value*, const std::string&, BasicBlock*);

void wrap_SetCondInst()
{
    // Name C++ overloads
    Instruction::BinaryOps (SetCondInst::*getInverseCondition)() const = &SetCondInst::getInverseCondition;
    Instruction::BinaryOps (*getInverseCondition_static)(Instruction::BinaryOps) = &SetCondInst::getInverseCondition;
    Instruction::BinaryOps (SetCondInst::*getSwappedCondition)() const = &SetCondInst::getSwappedCondition;
    Instruction::BinaryOps (*getSwappedCondition_static)(Instruction::BinaryOps) = &SetCondInst::getSwappedCondition;
    bool (*classof_SetCondInst)(const SetCondInst*) = &SetCondInst::classof;
    bool (*classof_Instruction)(const Instruction*) = &SetCondInst::classof;
    bool (*classof_Value)(const Value*) = &SetCondInst::classof;

    class_<SetCondInst, boost::noncopyable, bases<BinaryOperator> >
        ("SetCondInst", init<Instruction::BinaryOps, Value*, Value*,
                             optional<const std::string> >())
        .def("init", INST_INIT_DEF5(SetCondInst, Instruction::BinaryOps, Value*, Value*, const std::string&, Instruction*))
        .def("init", INST_INIT_DEF5(SetCondInst, Instruction::BinaryOps, Value*, Value*, const std::string&, BasicBlock*))
        .staticmethod("init")

        .def("getInverseCondition", getInverseCondition)
        .def("getInverseCondition", getInverseCondition_static)
        .staticmethod("getInverseCondition")
        .def("getSwappedCondition", getSwappedCondition)
        .def("getSwappedCondition", getSwappedCondition_static)
        .staticmethod("getSwappedCondition")
        .def("classof", classof_SetCondInst)
        .def("classof", classof_Instruction)
        .def("classof", classof_Value)
        .staticmethod("classof")
        ;
}

template CastInst* InstInitFn4(Value*, const Type*, const std::string&, Instruction*);
template CastInst* InstInitFn4(Value*, const Type*, const std::string&, BasicBlock*);

void wrap_CastInst()
{
    // Name C++ overloads
    bool (*classof_CastInst)(const CastInst*) = &CastInst::classof;
    bool (*classof_Instruction)(const Instruction*) = &CastInst::classof;
    bool (*classof_Value)(const Value*) = &CastInst::classof;

    class_<CastInst, boost::noncopyable, bases<Instruction> >
        ("CastInst", init<Value*, const Type*,
                          optional<const std::string> >())
        .def("init", INST_INIT_DEF4(CastInst, Value*, const Type*, const std::string&, Instruction*))
        .def("init", INST_INIT_DEF4(CastInst, Value*, const Type*, const std::string&, BasicBlock*))
        .staticmethod("init")

        .def("clone", &CastInst::clone,
             return_value_policy<manage_new_object>())
        .def("classof", classof_CastInst)
        .def("classof", classof_Instruction)
        .def("classof", classof_Value)
        .staticmethod("classof")
        ;
}

template CallInst* InstInitFn4(Value*, const std::vector<Value*>&, const std::string&, Instruction*);
template CallInst* InstInitFn4(Value*, const std::vector<Value*>&, const std::string&, BasicBlock*);
template CallInst* InstInitFn5(Value*, Value*, Value*, const std::string&, Instruction*);
template CallInst* InstInitFn5(Value*, Value*, Value*, const std::string&, BasicBlock*);
template CallInst* InstInitFn4(Value*, Value*, const std::string&, Instruction*);
template CallInst* InstInitFn4(Value*, Value*, const std::string&, BasicBlock*);
template CallInst* InstInitFn3(Value*, const std::string&, Instruction*);
template CallInst* InstInitFn3(Value*, const std::string&, BasicBlock*);

void wrap_CallInst()
{
    // Name C++ overloads
    const Value* (CallInst::*getCalledValue)() const = &CallInst::getCalledValue;
    bool (*classof_CallInst)(const CallInst*) = &CallInst::classof;
    bool (*classof_Instruction)(const Instruction*) = &CallInst::classof;
    bool (*classof_Value)(const Value*) = &CallInst::classof;

    class_<CallInst, boost::noncopyable, bases<Instruction> >
        ("CallInst", init<Value*, const std::vector<Value*>&,
                          optional<const std::string> >())
        .def(init<Value*, Value*, Value*,
                  optional<const std::string> >())
        .def(init<Value*, Value*,
                  optional<const std::string> >())
        .def(init<Value*,
                  optional<const std::string> >())
        .def("init", INST_INIT_DEF4(CallInst, Value*, const std::vector<Value*>&, const std::string&, Instruction*))
        .def("init", INST_INIT_DEF4(CallInst, Value*, const std::vector<Value*>&, const std::string&, BasicBlock*))
        .def("init", INST_INIT_DEF5(CallInst, Value*, Value*, Value*, const std::string&, Instruction*))
        .def("init", INST_INIT_DEF5(CallInst, Value*, Value*, Value*, const std::string&, BasicBlock*))
        .def("init", INST_INIT_DEF4(CallInst, Value*, Value*, const std::string&, Instruction*))
        .def("init", INST_INIT_DEF4(CallInst, Value*, Value*, const std::string&, BasicBlock*))
        .def("init", INST_INIT_DEF3(CallInst, Value*, const std::string&, Instruction*))
        .def("init", INST_INIT_DEF3(CallInst, Value*, const std::string&, BasicBlock*))
        .staticmethod("init")

        .def("clone", &CallInst::clone,
             return_value_policy<manage_new_object>())
        .def("mayWriteToMemory", &CallInst::mayWriteToMemory)
        .def("getCalledFunction", &CallInst::getCalledFunction,
             return_internal_reference<>())
        .def("getCalledValue", getCalledValue,
             return_internal_reference<>())
        .def("classof", classof_CallInst)
        .def("classof", classof_Instruction)
        .def("classof", classof_Value)
        .staticmethod("classof")
        ;
}

void wrap_Instructions1()
{
    wrap_AllocationInst();
    wrap_MallocInst();
    wrap_AllocaInst();
    wrap_FreeInst();
    wrap_LoadInst();
    wrap_StoreInst();
    wrap_GetElementPtrInst();
    wrap_SetCondInst();
    wrap_CastInst();
    wrap_CallInst();
}

}} // namespace llvm::Python

// EOF
