/*
 * Module.cpp: Python bindings for the LLVM compiler infrastructure
 * Copyright (c) 2004, 2005 Jarno Seppnen <--jseppanen@gmail.com-->.
 * Distributed under the Boost Software License, Version 1.0. (See
 * accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#include <boost/python.hpp>
#include <boost/noncopyable.hpp>
#include <llvm/Assembly/AsmAnnotationWriter.h>
#include <llvm/DerivedTypes.h> // FunctionType
#include <llvm/Module.h>
#include <llvm/SymbolTable.h>
#include <llvm/Type.h>

#include "scitbx/container_conversions.h"

using namespace boost::python;
using namespace scitbx::boost_python::container_conversions;

namespace llvm { namespace Python {

Function* getOrInsertFunction0(Module& Mod, const std::string& Name, const Type* RetTy)
{
    return Mod.getOrInsertFunction(Name, RetTy, 0);
}

Function* getOrInsertFunction1(Module& Mod, const std::string& Name, const Type* RetTy, const std::vector<const Type*>& ArgTys)
{
    return Mod.getOrInsertFunction(Name, FunctionType::get(RetTy, ArgTys, false));
}

void wrap_Module()
{
    // Name C++ overloads
    Function* (Module::*getOrInsertFunction)(const std::string&, const FunctionType*) = &Module::getOrInsertFunction;
    //Function* (Module::*getOrInsertFunction_vararg)(const std::string&, const Type*, ...) = &Module::getOrInsertFunction;
    const Module::GlobalListType& (Module::*getGlobalList)() const = &Module::getGlobalList;
    const Module::FunctionListType& (Module::*getFunctionList)() const = &Module::getFunctionList;
    const SymbolTable& (Module::*getSymbolTable)() const = &Module::getSymbolTable;

    Module::const_giterator (Module::*global_begin)() const = &Module::global_begin;
    Module::const_giterator (Module::*global_end)() const = &Module::global_end;
    Module::const_iterator (Module::*begin)() const = &Module::begin;
    Module::const_iterator (Module::*end)() const = &Module::end;
    void (Module::*print)(std::ostream&) const = &Module::print;
    void (Module::*print_AAW)(std::ostream&, AssemblyAnnotationWriter*) const = &Module::print;

    scope ModuleScope
        = class_<Module, std::auto_ptr<Module>, boost::noncopyable>
        ("Module", init<std::string>())

        .def("getModuleIdentifier", &Module::getModuleIdentifier,
             return_value_policy<return_by_value>())
        .def("getTargetTriple", &Module::getTargetTriple,
             return_value_policy<return_by_value>())
        .def("setTargetTriple", &Module::setTargetTriple)
        .def("getEndianness", &Module::getEndianness)
        .def("setEndianness", &Module::setEndianness)
        .def("getPointerSize", &Module::getPointerSize)
        .def("setPointerSize", &Module::setPointerSize)
        .def("getOrInsertFunction", getOrInsertFunction,
             return_internal_reference<>())
        //.def("getOrInsertFunction", getOrInsertFunction_vararg,
        //     return_internal_reference<>())
        .def("getOrInsertFunction", getOrInsertFunction0,
             return_internal_reference<>())
        .def("getOrInsertFunction", getOrInsertFunction1,
             return_internal_reference<>())
        .def("getFunction", &Module::getFunction,
             return_internal_reference<>())
        .def("getMainFunction", &Module::getMainFunction,
             return_internal_reference<>())
        .def("getNamedFunction", &Module::getNamedFunction,
             return_internal_reference<>())
        .def("getGlobalVariable", &Module::getGlobalVariable,
             return_internal_reference<>())
        .def("addTypeName", &Module::addTypeName)
        .def("getTypeName", &Module::getTypeName)
        .def("getTypeByName", &Module::getTypeByName,
             return_internal_reference<>())
        .def("getGlobalList", getGlobalList,
             return_value_policy<copy_const_reference>())
        .def("getFunctionList", getFunctionList,
             return_value_policy<copy_const_reference>())
        .def("getSymbolTable", getSymbolTable,
             return_internal_reference<>())
        .add_property("globals", range(global_begin, global_end))
        //.def("global_begin", global_begin)
        //.def("global_end", global_end)
        .def("global_empty", &Module::global_empty)
        .add_property("functions", range(begin, end))
        //.def("begin", begin)
        //.def("end", end)
        .def("size", &Module::size)
        .def("empty", &Module::empty)
        .add_property("libraries", range(&Module::lib_begin, &Module::lib_end))
        //.def("lib_begin", &Module::lib_begin)
        //.def("lib_end", &Module::lib_end)
        .def("lib_size", &Module::lib_size)
        .def("addLibrary", &Module::addLibrary)
        .def("removeLibrary", &Module::removeLibrary)
        .def("getLibraries", &Module::getLibraries,
             return_value_policy<copy_const_reference>())
        .def("print_", print)
        .def("print_", print_AAW)
        .def("dump", &Module::dump)
        //requires CopyConstructible? (lexical_cast)
        //.def(str(self)) //ostream& operator<<(ostream&, const Module&)
        .def("dropAllReferences", &Module::dropAllReferences)
        ;

    enum_<Module::Endianness>("Endianness")
        .value("AnyEndianness", Module::AnyEndianness)
        .value("LittleEndian", Module::LittleEndian)
        .value("BigEndian", Module::BigEndian)
        ;

    enum_<Module::PointerSize>("PointerSize")
        .value("AnyPointerSize", Module::AnyPointerSize)
        .value("Pointer32", Module::Pointer32)
        .value("Pointer64", Module::Pointer64)
        ;

    // Register conversions for tuple arguments
    from_python_sequence<std::vector<const Type*>,
                         variable_capacity_policy>();

    // Register conversions for tuple return values
    to_python_converter<iplist<GlobalVariable>,
                        to_tuple<iplist<GlobalVariable> > >();
    to_python_converter<iplist<Function>,
                        to_tuple<iplist<Function> > >();
    to_python_converter<SetVector<std::string>,
                        to_tuple<SetVector<std::string> > >();
}
}} // namespace llvm::Python

// EOF
